"use strict";
var commander = require("commander");
var fs = require("fs");
var glob = require("glob");
var request = require("request");
var templates_1 = require("./templates");
var introspectionQuery_1 = require("graphql/utilities/introspectionQuery");
exports.initCLI = function (args) {
    commander
        .version('0.0.1')
        .usage('graphql-codegen [options]')
        .option('-d, --dev', 'Turn on development mode - prints results to console')
        .option('-f, --file <filePath>', 'Parse local GraphQL introspection JSON file')
        .option('-u, --url <graphql-endpoint>', 'Parse remote GraphQL endpoint as introspection file')
        .option('-t, --template <template-name>', 'Language/platform name templates')
        .option('-o, --out <path>', 'Output file(s) path', String, './')
        .arguments('<options> [documents...]')
        .parse(args);
    return commander;
};
exports.cliError = function (err) {
    if (typeof err === 'object') {
        console.log(err);
    }
    console.error('Error: ' + err);
    process.exit(1);
    return;
};
exports.validateCliOptions = function (options) {
    var file = options['file'];
    var url = options['url'];
    var template = options['template'];
    var out = options['out'];
    var documents = options['args'] || [];
    if (!file && !url) {
        exports.cliError('Please specify one of --file or --url flags!');
    }
    if (!template) {
        exports.cliError('Please specify language/platform, using --template flag');
    }
};
var getGeneratorTemplate = function (templateName) {
    return templates_1.generators.find(function (item) { return (item.aliases || []).indexOf(templateName.toLowerCase()) > -1; });
};
exports.transformOptions = function (options) {
    var file = options['file'];
    var url = options['url'];
    var documents = options['args'] || [];
    var template = options['template'];
    var out = options['out'];
    var isDev = options['dev'] !== undefined;
    var result = {};
    var introspectionPromise;
    if (isDev) {
        console.log('Development mode is ON - output will print to console');
    }
    if (file) {
        introspectionPromise = new Promise(function (resolve, reject) {
            if (fs.existsSync(file)) {
                try {
                    var fileContent = fs.readFileSync(file, 'utf8');
                    if (!fileContent) {
                        reject("Unable to read local introspection file: " + file);
                    }
                    resolve(JSON.parse(fileContent));
                }
                catch (e) {
                    reject(e);
                }
            }
            else {
                reject("Unable to locate local introspection file: " + file);
            }
        });
    }
    else if (url) {
        introspectionPromise = new Promise(function (resolve, reject) {
            request.post({
                url: url,
                json: {
                    query: introspectionQuery_1.introspectionQuery
                },
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                }
            }, function (err, response, body) {
                if (err) {
                    reject(err);
                    return;
                }
                var bodyJson = body.data;
                if (!bodyJson || (bodyJson.errors && bodyJson.errors.length > 0)) {
                    reject('Unable to download schema from remote: ' + bodyJson.errors.map(function (item) { return item.message; }).join(', '));
                    return;
                }
                console.log(JSON.stringify(bodyJson));
                resolve(bodyJson);
            });
        });
    }
    var documentsPromises = documents.map(function (documentGlob) {
        return new Promise(function (resolve, reject) {
            glob(documentGlob, function (err, files) {
                if (err) {
                    reject(err);
                }
                if (!files || files.length === 0) {
                    reject("Unable to locate files matching glob definition: " + documentGlob);
                }
                resolve(files);
            });
        });
    });
    var generatorTemplatePromise = new Promise(function (resolve, reject) {
        var generatorTemplate = getGeneratorTemplate(template);
        if (generatorTemplate) {
            resolve(generatorTemplate);
        }
        else {
            var allowedTemplates = templates_1.generators.map(function (item) { return item.aliases; }).reduce(function (a, b) { return a.concat(b); }).join(', ');
            reject("Unknown template language specified: " + template + ", available are: " + allowedTemplates);
        }
    });
    var documentsPromise = Promise.all(documentsPromises).then(function (files) {
        return files.length === 0 ? [] : files.reduce(function (a, b) {
            return a.concat(b);
        });
    });
    return Promise.all([
        introspectionPromise,
        documentsPromise,
        generatorTemplatePromise
    ]).then(function (_a) {
        var introspection = _a[0], documents = _a[1], generatorTemplate = _a[2];
        result.introspection = introspection;
        result.documents = documents;
        result.template = generatorTemplate;
        result.outPath = out;
        result.isDev = isDev;
        return result;
    });
};
//# sourceMappingURL=cli.js.map