#!/usr/bin/env node
"use strict";
var cli_1 = require("./cli");
var scheme_loader_1 = require("./scheme-loader");
var codegen_1 = require("./codegen");
var document_loader_1 = require("./document-loader");
var generator_1 = require("./generator");
var fs = require("fs");
var path = require("path");
var mkdirp = require("mkdirp");
var handlebars_helpers_1 = require("./handlebars-helpers");
var options = cli_1.initCLI(process.argv);
cli_1.validateCliOptions(options);
cli_1.transformOptions(options)
    .then(function (transformedOptions) {
    handlebars_helpers_1.initHelpers();
    var schema = scheme_loader_1.loadSchema(transformedOptions.introspection);
    var documents = transformedOptions.documents;
    var codegen = codegen_1.prepareCodegen(schema, document_loader_1.loadDocumentsSources(documents), transformedOptions.template.config.primitives);
    var templateConfig = transformedOptions.template.config;
    var strategy = templateConfig.strategy || 'SINGLE_FILE';
    var baseOutPath = path.basename(transformedOptions.outPath);
    if (strategy === 'SINGLE_FILE') {
        if (baseOutPath.indexOf('.') === -1) {
            throw "Generator '" + transformedOptions.template.language + "' uses single-file strategy! Please specify a filename using --out flag!";
        }
        var templatePath = path.resolve(transformedOptions.template.config.basePath, transformedOptions.template.config.template);
        return [{
                isDev: transformedOptions.isDev,
                content: generator_1.generateCode(codegen, templatePath),
                path: path.resolve(transformedOptions.outPath)
            }];
    }
    else if (strategy === 'MULTIPLE_FILES') {
        if (baseOutPath.indexOf('.') > -1) {
            throw "Generator '" + transformedOptions.template.language + "' uses multiple-files strategy! Please specify a directory using --out flag!";
        }
        var resultsArr_1 = [];
        var filesExtension_1 = transformedOptions.template.config.filesExtension;
        var templates_1 = transformedOptions.template.config.templates;
        var outPath = path.resolve(transformedOptions.outPath);
        mkdirp.sync(outPath);
        Object.keys(templates_1).forEach(function (templateName) {
            var templatePath = path.resolve(transformedOptions.template.config.basePath, templates_1[templateName]);
            if (templateName === 'model') {
                codegen.models.forEach(function (model) {
                    resultsArr_1.push({
                        isDev: transformedOptions.isDev,
                        content: generator_1.generateCode(model, templatePath),
                        path: path.resolve(transformedOptions.outPath, model.name + '.model.' + filesExtension_1)
                    });
                });
            }
            if (templateName === 'document') {
                codegen.documents.forEach(function (document) {
                    resultsArr_1.push({
                        isDev: transformedOptions.isDev,
                        content: generator_1.generateCode(document, templatePath),
                        path: path.resolve(transformedOptions.outPath, document.name + '.document.' + filesExtension_1)
                    });
                });
            }
        });
        resultsArr_1 = resultsArr_1.filter(function (item) { return item.content.length > 0; });
        if (templates_1['index']) {
            var directoryPath_1 = path.resolve(transformedOptions.outPath);
            var indexOutPath = path.resolve(directoryPath_1, 'index.' + filesExtension_1);
            var templatePath = path.resolve(transformedOptions.template.config.basePath, templates_1['index']);
            resultsArr_1.push({
                isDev: transformedOptions.isDev,
                content: generator_1.generateCode({ files: resultsArr_1.map(function (item) {
                        return {
                            fileName: path.basename(item.path, '.' + filesExtension_1),
                            fullPath: item.path,
                            extension: filesExtension_1,
                            directory: directoryPath_1
                        };
                    }) }, templatePath),
                path: indexOutPath
            });
        }
        return resultsArr_1;
    }
    else {
        throw "Unknown strategy (" + strategy + ") specified in language template " + transformedOptions.template.language;
    }
})
    .then(function (generationResult) {
    generationResult.forEach(function (file) {
        if (file.isDev) {
            console.log("================== " + file.path + " ==================");
            console.log(file.content);
        }
        else {
            fs.writeFileSync(file.path, file.content);
            console.log("Generated file written to " + file.path);
        }
    });
})
    .catch(cli_1.cliError);
//# sourceMappingURL=index.js.map